<?php

namespace ACL\Repositories;

use ACL\Traits\TQuery;
use ACL\Helpers\Instances\InstanceProvider;
use Doctrine\DBAL\Exception;
use Doctrine\ORM\EntityManager;

class Filioper extends RepositoryBase {

    Use TQuery;

    protected $table = 'FILIOPER';

    /** @var EntityManager */
    protected $entityManager;

    public function __construct(EntityManager $entityManager)
    {
        parent::__construct($this->table);
        $this->entityManager = $entityManager;
    }

    /**
     * @param $filters
     * @return array[]
     * @throws Exception
     */
    public function findFilioper($filters): array
    {
        $query = <<<SQL
            SELECT DISTINCT CDOPERADOR, CDFILIAL, IDEXIBAVISO, NRORGFILIAL, DTULTATUFOPE, NRORG
                   DTINCLUSAO, DTULTATU, NRORGINCLUSAO, CDOPERINCLUSAO, NRORGULTATU, 
                   CDOPERULTATU, IDATIVO, NMFILIAL, TO_CHAR(DTVALIDADE, 'DD/MM/YYYY') AS DTVALIDADE, NRINSJURFILI,
                   NMPAIS, NMESTADO, NMMUNICIPIO, NMREGIAO
              FROM (SELECT CDOPERADOR, FIOP.CDFILIAL, IDEXIBAVISO, NRORGFILIAL, DTULTATUFOPE,
                           FIOP.NRORG, FIOP.DTINCLUSAO, FIOP.DTULTATU, FIOP.NRORGINCLUSAO,
                           FIOP.CDOPERINCLUSAO, FIOP.NRORGULTATU, FIOP.CDOPERULTATU, FIOP.IDATIVO,
                           FILI.NMFILIAL, FIOP.DTVALIDADE, FILI.NRINSJURFILI,
                            PA.NMPAIS, ES.NMESTADO, MU.NMMUNICIPIO, RE.NMREGIAO
                      FROM FILIOPER FIOP
                      JOIN FILIAL FILI ON FILI.NRORG = FIOP.NRORG
                                      AND FILI.CDFILIAL = FIOP.CDFILIAL
                      JOIN ENDEFILI EM 
                            ON FILI.CDFILIAL = EM.CDFILIAL
                            AND FILI.NRORG = EM.NRORG
                            AND EM.IDTPENDEFILI = 'P'
                    LEFT JOIN PAIS PA 
                            ON EM.CDPAIS = PA.CDPAIS
                            AND EM.NRORG = PA.NRORG
                    LEFT JOIN ESTADO ES
                            ON EM.SGESTADO = ES.SGESTADO
                            AND EM.CDPAIS = ES.CDPAIS
                            AND EM.NRORG = ES.NRORG
                    LEFT JOIN MUNICIPIO MU 
                            ON EM.CDMUNICIPIO = MU.CDMUNICIPIO
                           AND EM.SGESTADO = MU.SGESTADO
                           AND EM.CDPAIS = MU.CDPAIS
                           AND EM.NRORG = MU.NRORG
                    LEFT JOIN REGIAO RE 
                            ON FILI.CDREGIAO = RE.CDREGIAO
                            AND EM.SGESTADO = RE.SGESTADO
                            AND EM.CDPAIS = RE.CDPAIS
                            AND EM.NRORG = RE.NRORG)
                                      
              WHERE 1 = 1
              $filters
SQL;
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $query);
    }
}
