import { Http, IEventParam, Metadata, Singleton } from '@zeedhi/core';
import { TekGrid, TekRestDatasource } from '@zeedhi/teknisa-components-common';
import { Form, IGridEvent, LoadingService, Modal, ModalService, Select, Tabs, TextInput } from '@zeedhi/common';
import { OperadorService } from '@/services/OperadorService';
import AclMessages from '@/Util/AclMessages';
import { Date as DateInput } from '@zeedhi/common/types/components/zd-date/date';
import { PmLibHttp, PmLibAlertService, PmLibDialogService, PmLibUtil, PmLibFormSave } from '@zeedhi/pm-lib';

@Singleton
export class OperadorController {
	public operadorService!: OperadorService;

	public nmorganizacao = '';

	public nrorg = '';

	constructor() {
		this.operadorService = new OperadorService();
	}

	/** **********************************
	 ******* Operador Card Section *******
	*********************************** */

	private addOperadorModal!: Modal;

	public addOperadorModalPath: string = 'components/operador/grid/modais/addOperador';

	public async onCreatedOperadorCard() {
		PmLibUtil.validateNovocodigo();

		const response = await Http.get('/organizacao');
		if (response.data.data.length > 0) {
			const org = response.data.data[0];
			this.nmorganizacao = org.NMORGANIZACAO;
			this.nrorg = org.NRORG;
		}

		this.addOperadorModal = await ModalService.createFromJson(this.addOperadorModalPath, true);
	}

	public onBeforeDestroyOperadorCard() {
		this.addOperadorModal?.destroy();
	}

	/** **********************************
	 ******* Operador Grid Section *******
	 *********************************** */

	public isVisibleOperadorGrid = true;

	public readonlyNrorgoperFieldFilter = false;

	public currentOperador: any = {};

	public backupRow: any = {};

	public getInstanceOperadorGrid(): TekGrid {
		return Metadata.getInstance<TekGrid>('operadorGrid');
	}

	public async onMountedOperadorGrid() {
		const grid = this.getInstanceOperadorGrid();

		// const { data: { data } } = await Http.get('/findOrganizacaoRelacionadaByOperador');
		const { data: { data } } = await PmLibHttp.$get('/findOrganizacaoRelacionadaByOperador');

		if (grid.datasource instanceof TekRestDatasource) {
			grid.datasource.dynamicFilter = {
				IDATIVO: [
					{
						operation: 'EQUALS',
						relation: 'AND',
						value: 'S',
					},
				],
			};

			if (data.length === 1) {
				grid.datasource.dynamicFilter.NRORGOPER = [
					{
						operation: 'IN',
						relation: 'AND',
						value: [data[0].NRORG],
					},
				];
				this.readonlyNrorgoperFieldFilter = true;
			} else {
				this.readonlyNrorgoperFieldFilter = false;
			}

			try {
				await grid.datasource.get();
			} catch (e) {
				PmLibDialogService.showErrorBackend(e);
			}
		}
	}

	public addClickOperadorGrid() {
		this.addOperadorModal.show();
	}

	public async onMountedAddOperadorFormModal() {
		const form = Metadata.getInstance<Form>('addOperadorFormModal');
		const cdoperadorField = form.getChildInstance<TextInput>('CDOPERADOR');
		const cpfField = form.getChildInstance<TextInput>('NRINSCRICAOPARC');
		const cdpaisField = form.getChildInstance<Select>('CDPAIS');
		const nmorganizacaooperField = form.getChildInstance<Select>('NRORGOPER');

		try {
			LoadingService.show();
			const { data: { data } } = await Http.get('/paramgeral');
			this.operadorService.setConfigCdoperadorAddField(data, cdoperadorField);
			this.operadorService.setConfigCpfAddField(data, cpfField);
			await cdpaisField.setValue('0055');
			await this.operadorService.setConfigNmorganizacaooperAddField(nmorganizacaooperField, this.nrorg);
			LoadingService.hide();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}

	public changeFieldCdoperador({ component }: IEventParam<TextInput>) {
		const form = Metadata.getInstance<Form>('addOperadorFormModal');
		// eslint-disable-next-line no-restricted-globals
		if (!isNaN(component.value)) {
			form.value.CDOPERADOR = PmLibUtil.fill(component.value, 12, false, false);
		}
	}

	public closeAddOperadorModal() {
		this.addOperadorModal.hide();
	}

	public async saveAddOperadorModal() {
		const addForm = Metadata.getInstance<PmLibFormSave>('addOperadorFormModal');
		const operadorGrid = this.getInstanceOperadorGrid();

		if (addForm.validate()) {
			addForm.value.NRINSCRICAOPARC = PmLibUtil.getOnlyNumberInString(addForm.value.NRINSCRICAOPARC);
			addForm.value.NRPERFILACESSO = localStorage.getItem('CDPERFILSELECIONADO');
			addForm.value.CDTIPOINSCRICAO = this.operadorService.getInscriptionType(addForm.value.NRINSCRICAOPARC);
			addForm.value.SHOW_FULL_OPERATOR = (window.showFullOperatorWindow || false);
			addForm.value.IDOPERATIVO = (addForm.value.IDATIVO === 'S') ? 'A' : 'I';
			addForm.value.ADMINISTRATOR = false;

			await PmLibHttp.$post('/operador', addForm.value);

			this.closeAddOperadorModal();
			operadorGrid.reload();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
		}
	}

	public async rowClickOperadorGrid({ row }: IGridEvent) {
		this.currentOperador = row;
		this.isVisibleOperadorGrid = false;
		if (!row.NRPARCNEGOCIO) {
			await this.operadorService.createParcnegocio(row);
		}
		this.setOperadorFormEdit(row);
		this.backupRow = PmLibUtil.clone(row);
		this.changeOperadorDetailTabs();
	}

	private setOperadorFormEdit(row: object) {
		const operadorFormEdit = Metadata.getInstance<PmLibFormSave>('operadorFormEdit');
		operadorFormEdit.value = row;
	}

	/** **********************************
	 ***** Operador Detail Section ******
	 *********************************** */

	get isVisibleOperadorDetail(): boolean {
		return !this.isVisibleOperadorGrid;
	}

	get currentOperadorName(): string {
		return this.currentOperador.NMOPERADOR || '';
	}

	public async clickBackToOperadorGrid() {
		this.isVisibleOperadorGrid = true;
		await this.getInstanceOperadorGrid().reload();
	}

	public async changeOperadorDetailTabs() {
		const tabs = Metadata.getInstance<Tabs>('operadorDetailTabs');
		if (tabs.activeTab === 1) {
			await this.operadorService.clickTabOperadorPerfilAcesso(this.currentOperador);
		}
	}

	/** **********************************
	 ***** Operador Edit Tab Section ******
	 *********************************** */

	public getOperadorFormEdit(): PmLibFormSave {
		return Metadata.getInstance<PmLibFormSave>('operadorFormEdit');
	}

	public async saveEditOperador() {
		const form = this.getOperadorFormEdit();
		if (form.validate()) {
			form.value.NRINSCRICAOPARC = PmLibUtil.getOnlyNumberInString(form.value.NRINSCRICAOPARC);
			form.value.NRPERFILACESSO = localStorage.getItem('CDPERFILSELECIONADO');
			form.value.CDTIPOINSCRICAO = this.operadorService.getInscriptionType(form.value.NRINSCRICAOPARC);
			form.value.SHOW_FULL_OPERATOR = (window.showFullOperatorWindow || false);
			form.value.IDOPERATIVO = (form.value.IDATIVO === 'S') ? 'A' : 'I';
			form.value.ADMINISTRATOR = false;

			form.value.OLD_CPF = this.backupRow.NRINSCRICAOPARC;
			form.value.OLD_CDCLIENTE = this.backupRow.CDCLIENTE;
			form.value.OLD_NMOPERADOR = this.backupRow.NMOPERADOR;

			await PmLibHttp.$put('/operador', form.value);
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_SAVE_DEAFULT);
		}
	}

	/** ***********************************
	 **** Perfil de Acesso Tab Section ****
	 ************************************ */

	/** Grid Component */

	public getCurrentNrparcnegocio(): string {
		return this.currentOperador.NRPARCNEGOCIO;
	}

	public async clickTab() {
		this.operadorService.clickTabOperadorPerfilAcesso(this.currentOperador);
	}

	private getInstanceOperadorPerfilDeAcessoGrid() {
		return Metadata.getInstance<TekGrid>('operadorPerfilDeAcessoGrid');
	}

	private getInstanceAddPerfilByOperadorGrid() {
		return Metadata.getInstance<TekGrid>('addPerfilByOperadorGrid');
	}

	public async addClickOperadorPerfilDeAcessoGrid() {
		await this.createAddPerfilByOperadorModal();
		await this.addPerfilAcessoByOperadorModal.show();
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		const nrparcnegocio = this.getCurrentNrparcnegocio();
		await addGrid.datasource.addFilter('NRPARCNEGOCIO', nrparcnegocio);
	}

	public async beforeDeleteOperadorPerfilDeAcessoGrid({ event }: IGridEvent) {
		event?.preventDefault();
		const confirm = await PmLibDialogService.confirmDialog(AclMessages.DELETE_DEFAULT);
		if (confirm) {
			this.deletePerfilacesso();
		}
	}

	public async deletePerfilacesso() {
		const grid = this.getInstanceOperadorPerfilDeAcessoGrid();
		await PmLibHttp.$delete('/perfacessparc', { data: grid.selectedRows });
		PmLibAlertService.showSuccess(AclMessages.SUCCESS_DELETE_DEAFULT);
		await grid.reload();
	}

	/** Form Add Modal Component */

	public addPerfilAcessoByOperadorModal!: Modal;

	public addPerfilAcessoByOperadorModalPath: string = 'components/operador/detail/tabs/operadorPerfilDeAcesso/modais/addPerfilByOperador';

	public async createAddPerfilByOperadorModal() {
		this.addPerfilAcessoByOperadorModal = await ModalService.createFromJson(this.addPerfilAcessoByOperadorModalPath, true);
	}

	public async closeAddPerfilByOperadorModal() {
		this.addPerfilAcessoByOperadorModal.hide();
		this.addPerfilAcessoByOperadorModal?.destroy();
		await this.getInstanceOperadorPerfilDeAcessoGrid().reload();
	}

	public get validateSaveButtonAddPerfilByOperadorModal(): boolean {
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		return !addGrid.selectedRows.length;
	}

	public async saveAddOperadorByPerfilModal() {
		const addGrid = this.getInstanceAddPerfilByOperadorGrid();
		const addPerfilModalForm = Metadata.getInstance<Form>('addPerfilByOperadorForm');
		const dtValidateField = addPerfilModalForm.getChildInstance<DateInput>('DTVALIDATE');
		const { selectedRows } = addGrid;
		selectedRows.forEach((row) => {
			row.DTVALIDADE = dtValidateField.value;
			row.NRPARCNEGOCIO = this.getCurrentNrparcnegocio();
			row.CDOPERADOR = this.currentOperador.CDOPERADOR;
			row.NMOPERADOR = this.currentOperador.NMOPERADOR;
		});

		try {
			LoadingService.show();
			await Http.post('/perfacessparc', selectedRows);
			LoadingService.hide();
			PmLibAlertService.showSuccess(AclMessages.SUCCESS_ADD_OPERADORES);
			dtValidateField.setDateValue('');
			addGrid.selectedRows = [];
			await addGrid.reload();
		} catch (e) {
			LoadingService.hide();
			PmLibDialogService.showErrorBackend(e);
		}
	}
}
